/**************************************************************/
/* BNC header file                                            */
/* Copyright (c) 2000- Tomonori Kouya                         */
/* History:                                                   */
/* Version 0.0.1  2000.5/23: modifying mpf_sin, mpf_cos       */
/*         0.0.2  2000.5/26: append mpf_ln, mpf_log10         */
/*         0.0.3  2000.6/13: append FStack, DStack, MPFStack  */
/*         0.0.4  2000.6/14: append and test euler.c ex_nim.c */
/*         0.0.5  2000.6/19: append and test newton.c         */
/*         0.0.6  2000.6/19: append and test rk.c             */
/*         0.0.7  2000.6/20: append set0_[f,d,mpf]vector      */
/*         0.0.7a 2000.6/20: append set0_[f,d,mpf]matrix      */
/*                           append setI_[f,d,mpf]matrix      */
/*         0.0.7b 2000.7/04: revise lu.c                      */
/*         0.0.7c 2000.7/05: revise euler.c, rk.c, rkcoef.c   */
/*         0.0.7d 2000.12/4: append power.c                   */
/*         0.0.7e 2001.4/23: append integral.c                */
/*         0.0.7f 2001.4/24: append bnc_mpf_[init]_set_d      */
/*         0.0.7g 2001.6/05: append ivpower.c                 */
/*         0.1    2001.8/02: setting precision for            */
/*                           MPF* functions                   */
/*         0.1a   2001.8/08: Bug Fix : init_*vector[2],       */
/*                           rkcoef.c, ex_nim.c               */
/*         0.2    2002.5/04: append poly.c, complex.c,        */
/*                           array, dka.c                     */
/*         0.2a   2002.5/28: append iterative.c               */
/*         0.3    2002.9/05: include mpfr                     */
/*         0.3a   2003.2/27: append get_sec.c & bug fix       */
/*         0.3b   2003.2/27: separate modules of ODEs         */
/*         0.3c   2003.6/11: append qr.c & gtestmat.c         */
/*         0.3d   2003.12/6: append get_real_sec()            */
/*         0.3e   2004.1/07: include new functions            */
/*                                       from mpfr-2.0.2      */
/*         0.3e1  2006.1/07: applying Paul Zemmerman's patch  */
/*                           (Thanks !)                       */
/*                                                            */
/* This library is free software; you can re- distribute it   */
/* and/or modify it under the terms of the GNU Lesser General */
/* Public License as published by the Free Software           */
/* Foundation; either version 2.1 of the License, or (at your */
/* option) any later version.                                 */
/*                                                            */
/* This library is distributed in the hope that it will be    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR    */
/* PURPOSE.  See the GNU Lesser General Public License for    */
/* more details.                                              */
/**************************************************************/
#ifndef __STDIO_H__
#include <stdio.h>
#endif
#ifndef __BNC_H
#define __BNC_H

#ifdef USE_GMP
  #ifndef __GMP_H
    #include "gmp.h"
//    #include "gmp-impl.h"
  #endif
  static unsigned long int bnc_default_prec = 128; /* Global variable */
  #ifdef USE_MPFR
    #ifndef __MPFR_H
      #include "mpfr.h"
//      #include "mpfr-impl.h"
      #include "_bnc_mpf.h" // mpf_* -> _bnc_mpf_*
      #include "mpf2mpfr.h"
// check MPFR's Version 
      #ifdef mpfr_cmp_abs // over 2.0.2
	#define _BNC_MPFR_VER 202
      #else
	#define _BNC_MPFR_VER 201 // mpfr in gmp-4.1.2
      #endif
      #define mpf_ptr mpfr_ptr
      static unsigned long int bnc_default_rounding_mode = GMP_RNDN; /* Global variable */
    #endif
  #endif
#endif

#if defined (__cplusplus)
extern "C" {
#endif
/*************************************************/
/* double get_sec(int flag)                      */
/*************************************************/
double get_sec(int);
double get_secv(void);
float fget_sec(int);
float fget_secv(void);

double get_real_sec(int);
double get_real_secv(void);
float fget_real_sec(int);
float fget_real_secv(void);

/*************************************************/
/* Complex -> Cmplx                              */
/* FCmplx, DCmplx, MPFCmplx                      */
/*************************************************/
typedef struct{
	float re;
	float im;
} fcmplx;

typedef fcmplx *FCmplx;

typedef struct{
	double re;
	double im;
} dcmplx;

typedef dcmplx *DCmplx;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
	mpf_t re;
	mpf_t im;
} mpfcmplx;

typedef mpfcmplx *MPFCmplx;
#endif

/*************************************************/
/* Polynomial Type: FPoly, DPoly, MPFPoly        */
/*                  CFPoly, CDPoly, CMPFPoly     */
/*************************************************/
typedef struct{
	float *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} fpoly;

typedef fpoly *FPoly;

typedef struct{
	fcmplx *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} cfpoly;

typedef cfpoly *CFPoly;

typedef struct{
	double *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} dpoly;

typedef dpoly *DPoly;

typedef struct{
	dcmplx *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} cdpoly;

typedef cdpoly *CDPoly;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
	mpf_t *coef;
	mpf_t zero; /* ... + 0 x^(deg+2) + 0 x^(deg+1) + coef[i] x^deg + ... */
//	mpf_ptr *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} mpfpoly;

typedef mpfpoly *MPFPoly;

typedef struct{
	unsigned long int prec;
	mpfcmplx *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} cmpfpoly;

typedef cmpfpoly *CMPFPoly;
#endif

/*************************************************/
/* Vector Type: FVector, DVector, MPFVector      */
/*              CFVector, CDVector, CMPFVector   */
/*************************************************/
typedef struct{
	float *element;
	long int dim;
} fvector;

typedef fvector *FVector;

typedef struct{
	fcmplx *element;
	long int dim;
} cfvector;

typedef cfvector *CFVector;

typedef struct{
	double *element;
	long int dim;
} dvector;

typedef dvector *DVector;

typedef struct{
	dcmplx *element;
	long int dim;
} cdvector;

typedef cdvector *CDVector;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
	mpf_t *element;
//	mpf_ptr *element;
	long int dim;
} mpfvector;

typedef mpfvector *MPFVector;

typedef struct{
	unsigned long int prec;
	mpfcmplx *element;
	long int dim;
} cmpfvector;

typedef cmpfvector *CMPFVector;
#endif

/*************************************************/
/* Matrix Type: FMatrix, DMatrix, MPFMatrix      */
/*              CFMatrix, CDMatrix, CMPFMatrix   */
/*************************************************/
#define MTYPE_DEFAULT		0x3F /* 0011 1111 square, unsymmetric, dense */
#define MTYPE_GENERAL		0x00 /*        00 */
#define MTYPE_SQUARE		0x03 /*        11 */
#define MTYPE_SYMMETRIC		0x00 /*      00   */
#define MTYPE_UNSYMMETRIC	0x0C /*      11   */
#define MTYPE_SPARSE		0x00 /*   00      */
#define MTYPE_BAND		0x20 /*   10      */
#define MTYPE_DENSE		0x30 /*   11      */

typedef struct{
//	unsigned int type; /* Matrix Type */
	float *element;	   /* Elements */
//	long int element_size; /* Number of Elements */
	long int row_dim, col_dim; /* Number of Rows, Number of Columns */
//	long int *row, *col; /* Coordinates of ALL Elements */
} fmatrix;

typedef fmatrix *FMatrix;

typedef struct{
	fcmplx *element;	   /* Elements */
	long int row_dim, col_dim; /* Number of Rows, Number of Columns */
} cfmatrix;

typedef cfmatrix *CFMatrix;

typedef struct{
//	unsigned int type;
	double *element;
	long int row_dim, col_dim;
} dmatrix;

typedef dmatrix *DMatrix;

typedef struct{
	dcmplx *element;
	long int row_dim, col_dim;
} cdmatrix;

typedef cdmatrix *CDMatrix;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
//	unsigned int type;
	mpf_t *element;
//	mpf_ptr *element;
	long int row_dim, col_dim;
} mpfmatrix;

typedef mpfmatrix *MPFMatrix;

typedef struct{
	unsigned long int prec;
	mpfcmplx *element;
	long int row_dim, col_dim;
} cmpfmatrix;

typedef cmpfmatrix *CMPFMatrix;
#endif

/*************************************************/
/* Symmetric Matrix Type:                        */
/*              FSMatrix, DSMatrix, MPFSMatrix   */
/*************************************************/
typedef struct{
//	unsigned int type;
	float *element;
	long int dim;
} fsmatrix;

typedef fmatrix *FSMatrix;

typedef struct{
//	unsigned int type;
	double *element;
	long int dim;
} dsmatrix;

typedef dmatrix *DSMatrix;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
//	unsigned int type;
	mpf_t *element;
//	mpf_ptr *element;
	long int dim;
} mpfsmatrix;

typedef mpfmatrix *MPFSMatrix;
#endif

/*************************************************/
/* Band Matrix Type:                             */
/*              FBMatrix, DBMatrix, MPFBMatrix   */
/*************************************************/
typedef struct{
//	unsigned int type;
	float *element;
	long int dim, right_band_width, left_bandwidth;
} fbmatrix;

typedef fmatrix *FBMatrix;

typedef struct{
//	unsigned int type;
	double *element;
	long int dim, right_band_width, left_bandwidth;
} dbmatrix;

typedef dmatrix *DBMatrix;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
//	unsigned int type;
	mpf_t *element;
//	mpf_ptr *element;
	long int dim, right_band_width, left_bandwidth;
} mpfbmatrix;

typedef mpfmatrix *MPFBMatrix;
#endif

/*************************************************/
/* Symmetric Band Matrix Type:                   */
/*             FSBMatrix, DSBMatrix, MPFSBMatrix */
/*************************************************/
typedef struct{
//	unsigned int type;
	float *element;
	long int dim, band_width;
} fsbmatrix;

typedef fmatrix *FSBMatrix;

typedef struct{
//	unsigned int type;
	double *element;
	long int dim, band_width;
} dsbmatrix;

typedef dmatrix *DSBMatrix;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
//	unsigned int type;
	mpf_t *element;
//	mpf_ptr *element;
	long int dim, band_width;
} mpfsbmatrix;

typedef mpfmatrix *MPFSBMatrix;
#endif

/*************************************************/
/* Stack                                         */
/*************************************************/
/* fstack */
typedef struct {
	float *array; /* stack */
	long int size; /* Height of stack */
	long index; /* pointer to top of stack */
} fstack;

typedef fstack *FStack;

/* dstack */
typedef struct {
	double *array; /* stack */
	long int size; /* Height of stack */
	long index; /* pointer to top of stack */
} dstack;

typedef dstack *DStack;

#ifdef USE_GMP
/* mpfstack */
typedef struct {
	unsigned long prec; /* precision */
	mpf_t *array; /* stack */
	long int size; /* Height of stack */
	long index; /* pointer to top of stack */
} mpfstack;

typedef mpfstack *MPFStack;
#endif

/* errors */
// #define STACK_OVERFLOW	10
// #define STACK_UNDERFLOW	20

/*************************************************/
/* Array                                         */
/*************************************************/
/* farray */
typedef struct {
	float *array; /* array */
	long int size; /* Length of array */
} farray;

typedef farray *FArray;

/* cfarray */
typedef struct {
	fcmplx *array; /* array */
	long int size; /* Length of array */
} cfarray;

typedef cfarray *CFArray;

/* darray */
typedef struct {
	double *array; /* array */
	long int size; /* Length of array */
} darray;

typedef darray *DArray;

/* cdarray */
typedef struct {
	dcmplx *array; /* array */
	long int size; /* Length of array */
} cdarray;

typedef cdarray *CDArray;

#ifdef USE_GMP
/* mpfarray */
typedef struct {
	unsigned long prec; /* preision */
	mpf_t *array; /* array */
	long int size; /* Length of array */
} mpfarray;

typedef mpfarray *MPFArray;

/* mpfarray */
typedef struct {
	unsigned long prec; /* preision */
	mpfcmplx *array; /* array */
	long int size; /* Length of array */
} cmpfarray;

typedef cmpfarray *CMPFArray;
#endif

/*************************************************/
/* efunc.c: Elementary Functions                 */
/*************************************************/
long int lmax(long, long);
long int lmin(long, long);
long int lfact(long, long);
#if __GNUC__ < 3
float fmax(float, float);
float fmin(float, float);
#endif
float fpower(float, long);
double dmax(double, double);
double dmin(double, double);
double dpower(double, long);
#ifdef USE_GMP
void set_bnc_default_prec(unsigned long);
void set_bnc_default_prec_decimal(unsigned long);
void _set_bnc_default_prec(unsigned long);
void _set_bnc_default_prec_decimal(unsigned long);
unsigned long get_bnc_default_prec(void);
#ifdef USE_MPFR
  void set_bnc_rounding_mode(mp_rnd_t);
  void _set_bnc_rounding_mode(mp_rnd_t);
  mp_rnd_t get_bnc_default_rounding_mode(void);
#endif
void mpf_power(mpf_t, mpf_t, long);
mpf_ptr mpf_max(mpf_t, mpf_t);
mpf_ptr mpf_min(mpf_t, mpf_t);
void bnc_mpf_set_d(mpf_t, double);
void bnc_mpf_init_set_d(mpf_t, double);
#if (__GNU_MP_VERSION < 3)
void mpf_floor(mpf_t, mpf_t);
#else
  #ifdef USE_MPFR
    #define mpf2float(x) (float)mpfr_get_d(x, bnc_default_rounding_mode)
    #define mpf2double(x) mpfr_get_d(x, bnc_default_rounding_mode)
  #else
    float mpf2float(mpf_t);
    double mpf2double(mpf_t);
  #endif
#endif
#ifdef USE_MPFR
  #define mpf_pi(x) mpfr_const_pi(x, bnc_default_rounding_mode)
  void mpf_e(mpfr_t);
  /*  int mpfr_get_f(mpf_ptr, mpfr_srcptr, mpfr_rnd_t); */
  #define mpf_sin(x, y) mpfr_sin(x, y, bnc_default_rounding_mode)
  #define mpf_cos(x, y) mpfr_cos(x, y, bnc_default_rounding_mode)
  #define mpf_exp(x, y) mpfr_exp(x, y, bnc_default_rounding_mode)
  #define mpf_ln_2(x) mpfr_const_log2(x, bnc_default_rounding_mode)
  #define mpf_ln(x, y) mpfr_log(x, y, bnc_default_rounding_mode)
  #define mpf_log10(x, y) mpfr_log10(x, y, bnc_default_rounding_mode)
  #define mpf_euler(x) mpfr_const_euler(x, bnc_default_rounding_mode)
  #if ( _BNC_MPFR_VER > 201 )
      #define mpf_cbrt(x, y) mpfr_cbrt(x, y, bnc_default_rounding_mode)
      #define mpf_sin_cos(x, y, z) mpfr_sin_cos(x, y, z, bnc_default_rounding_mode)
      #define mpf_expm1(x, y) mpfr_expm1(x, y, bnc_default_rounding_mode)
      #define mpf_gamma(x, y) mpfr_gamma(x, y, bnc_default_rounding_mode)
      #define mpf_zeta(x, y) mpfr_zeta(x, y, bnc_default_rounding_mode)
      #define mpf_erf(x, y) mpfr_erf(x, y, bnc_default_rounding_mode)
  #endif  
#else
  void mpf_pi(mpf_t);
  void mpf_e(mpf_t);
  void mpf_sin(mpf_t, mpf_t);
  void mpf_cos(mpf_t, mpf_t);
  void mpf_exp(mpf_t, mpf_t);
  void mpf_ln_2(mpf_t);
  void mpf_ln(mpf_t, mpf_t);
  void mpf_log10(mpf_t, mpf_t);
#endif
#endif

/*************************************************/
/* stack.c: Stack                                */
/*************************************************/
FStack init_fstack(long int stack_size);
void free_fstack(FStack st);
void push_fstack(FStack st, float val);
float pop_fstack(FStack st);
DStack init_dstack(long int stack_size);
void free_dstack(DStack st);
void push_dstack(DStack st, double val);
double pop_dstack(DStack st);
#ifdef USE_GMP
MPFStack init_mpfstack(long int stack_size);
MPFStack init2_mpfstack(long int stack_size, unsigned long prec);
void free_mpfstack(MPFStack st);
void push_mpfstack(MPFStack st, mpf_t val);
void pop_mpfstack(mpf_t rval, MPFStack st);
#endif

/*************************************************/
/* array.c: array                                */
/*************************************************/
FArray init_farray(long int);
void free_farray(FArray);
float get_farray_i(FArray, long int);
void set_farray_i(FArray, long int, float);
void subst_farray(FArray, FArray);
void print_farray(FArray);

CFArray init_cfarray(long int);
void free_cfarray(CFArray);
FCmplx get_cfarray_i(CFArray, long int);
void set_cfarray_i(CFArray, long int, FCmplx);
void subst_cfarray(CFArray, CFArray);
void print_cfarray(CFArray);

DArray init_darray(long int);
void free_darray(DArray);
double get_darray_i(DArray, long int);
void set_darray_i(DArray, long int, double);
void subst_darray(DArray, DArray);
void print_darray(DArray);

CDArray init_cdarray(long int);
void free_cdarray(CDArray);
DCmplx get_cdarray_i(CDArray, long int);
void set_cdarray_i(CDArray, long int, DCmplx);
void subst_cdarray(CDArray, CDArray);
void print_cdarray(CDArray);

#ifdef USE_GMP
MPFArray init_mpfarray(long int);
MPFArray init2_mpfarray(long int, unsigned long);
void free_mpfarray(MPFArray);
mpf_ptr get_mpfarray_i(MPFArray, long int);
void set_mpfarray_i(MPFArray, long int, mpf_t);
void subst_mpfarray(MPFArray, MPFArray);
void print_mpfarray(MPFArray);

CMPFArray init_cmpfarray(long int);
CMPFArray init2_cmpfarray(long int, unsigned long);
void free_cmpfarray(CMPFArray);
MPFCmplx get_cmpfarray_i(CMPFArray, long int);
void set_cmpfarray_i(CMPFArray, long int, MPFCmplx);
void subst_cmpfarray(CMPFArray, CMPFArray);
void print_cmpfarray(CMPFArray);
#endif

/*************************************************/
/* linear.c: Function for Vector Types           */
/*************************************************/

/* Initialize:                                   */
FVector init_fvector(long int dimension);
DVector init_dvector(long int dimension);
#ifdef USE_GMP
MPFVector init_mpfvector(long int dimension);
MPFVector init2_mpfvector(long int dimension, unsigned long);
#endif

/* Free:                                         */
void free_fvector(FVector vec);
void free_dvector(DVector vec);
#ifdef USE_GMP
void free_mpfvector(MPFVector vec);
#endif

/* Get & Set Values:                             */
#define gfvi get_fvector_i
#define gdvi get_dvector_i
float get_fvector_i(FVector vec, long int index);
double get_dvector_i(DVector vec, long int index);
#ifdef USE_GMP
#define gmpfvi get_mpfvector_i
mpf_ptr get_mpfvector_i(MPFVector vec, long int index);
#endif
#define sfvi set_fvector_i
#define sdvi set_dvector_i
void set_fvector_i(FVector vec, long int index, float val);
void set_dvector_i(DVector vec, long int index, double val);
#ifdef USE_GMP
#define smpfvi set_mpfvector_i
#define smpfvid set_mpfvector_i_d
#define smpfvis set_mpfvector_i_str
#define smpfviui set_mpfvector_i_ui
void set_mpfvector_i(MPFVector vec, long int index, mpf_t val);
void set_mpfvector_i_d(MPFVector vec, long int index, double val);
void set_mpfvector_i_str(MPFVector vec, long int index, const char *str, int base);
void set_mpfvector_i_ui(MPFVector vec, long int index, unsigned long val);
#endif

/* For MPFVector */
#ifdef USE_GMP
unsigned long int prec_mpfvector(MPFVector);
unsigned long int minprec_mpfvector(MPFVector);
unsigned long int maxprec_mpfvector(MPFVector);
#endif

/* Output:                                       */
void print_fvector(FVector);
void print_dvector(DVector);
void print_fdvector(FVector, DVector);
#ifdef USE_GMP
void print_mpfvector(MPFVector);
void print_fdmpfvector(FVector, DVector, MPFVector);
#endif

/*************************************************/
/* linear.c: Function for Matrix Types           */
/*************************************************/

/* Initialize:                                   */
FMatrix init_fmatrix(long int row_dimension, long int col_dimension);
DMatrix init_dmatrix(long int row_dimension, long int col_dimension);
#ifdef USE_GMP
MPFMatrix init_mpfmatrix(long int row_dimension, long int col_dimension);
MPFMatrix init2_mpfmatrix(long int row_dimension, long int col_dimension, unsigned long mbits);
#endif

/* Free:                                         */
void free_fmatrix(FMatrix mat);
void free_dmatrix(DMatrix mat);
#ifdef USE_GMP
void free_mpfmatrix(MPFMatrix mat);
#endif

/* Get & Set:                                    */
#define gfmij get_fmatrix_ij
#define gdmij get_dmatrix_ij
float get_fmatrix_ij(FMatrix mat, long int row_index, long int col_index);
double get_dmatrix_ij(DMatrix mat, long int row_index, long int col_index);
#ifdef USE_GMP
#define gmpfmij get_mpfmatrix_ij
mpf_ptr get_mpfmatrix_ij(MPFMatrix mat, long int row_index, long int col_index);
#endif
#define sfmij set_fmatrix_ij
#define sdmij set_dmatrix_ij
void set_fmatrix_ij(FMatrix mat, long int row_index, long int col_index, float val);
void set_dmatrix_ij(DMatrix mat, long int row_index, long int col_index, double val);
#ifdef USE_GMP
#define smpfmij set_mpfmatrix_ij
#define smpfmijd set_mpfmatrix_ij_d
#define smpfmijs set_mpfmatrix_ij_str
#define smpfmijui set_mpfmatrix_ij_ui
void set_mpfmatrix_ij(MPFMatrix mat, long int row_index, long int col_index, mpf_t val);
void set_mpfmatrix_ij_d(MPFMatrix mat, long int row_index, long int col_index, double val);
void set_mpfmatrix_ij_str(MPFMatrix mat, long int row_index, long int col_index, const char *str, int base);
void set_mpfmatrix_ij_ui(MPFMatrix mat, long int row_index, long int col_index, const unsigned long val);
#endif

/* For MPFVector */
#ifdef USE_GMP
unsigned long int prec_mpfmatrix(MPFMatrix);
unsigned long int minprec_mpfmatrix(MPFMatrix);
unsigned long int maxprec_mpfmatrix(MPFMatrix);
#endif

/* Output:                                       */
void print_fmatrix(FMatrix mat);
void print_dmatrix(DMatrix mat);
#ifdef USE_GMP
void print_mpfmatrix(MPFMatrix mat);
#endif


/*************************************************/
/* linear.c: Elementary Arithmetic for Vector    */
/*************************************************/
/* For FVector */
void add_fvector(FVector, FVector, FVector);
void add2_fvector(FVector, FVector);
void sub_fvector(FVector, FVector, FVector);
void sub2_fvector(FVector, FVector);
void cmul_fvector(FVector, float, FVector);
void cmul2_fvector(FVector, float);
float ip_fvector(FVector, FVector);
float norm1_fvector(FVector);
float norm2_fvector(FVector);
float normi_fvector(FVector);
void subst_fvector(FVector, FVector);

/* For DVector */
void add_dvector(DVector, DVector, DVector);
void add2_dvector(DVector, DVector);
void sub_dvector(DVector, DVector, DVector);
void sub2_dvector(DVector, DVector);
void cmul_dvector(DVector, double, DVector);
void cmul2_dvector(DVector, double);
double ip_dvector(DVector, DVector);
double norm1_dvector(DVector);
double norm2_dvector(DVector);
double normi_dvector(DVector);
void subst_dvector(DVector, DVector);

/* For MPFVector */
#ifdef USE_GMP
void add_mpfvector(MPFVector, MPFVector, MPFVector);
void add2_mpfvector(MPFVector, MPFVector);
void sub_mpfvector(MPFVector, MPFVector, MPFVector);
void sub2_mpfvector(MPFVector, MPFVector);
void cmul_mpfvector(MPFVector, mpf_t, MPFVector);
void cmul2_mpfvector(MPFVector, mpf_t);
void ip_mpfvector(mpf_t, MPFVector, MPFVector);
void norm1_mpfvector(mpf_t, MPFVector);
void norm2_mpfvector(mpf_t, MPFVector);
void normi_mpfvector(mpf_t, MPFVector);
void subst_mpfvector(MPFVector, MPFVector);
#endif

/*************************************************/
/* linear.c: Elementary Arithmetic for Matrices  */
/*************************************************/
/* For FMatrix */
void add_fmatrix(FMatrix, FMatrix, FMatrix);
void sub_fmatrix(FMatrix, FMatrix, FMatrix);
void subst_fmatrix(FMatrix, FMatrix);
void set0_fmatrix(FMatrix);
void setI_fmatrix(FMatrix);
void cmul_fmatrix(FMatrix, float, FMatrix);
void mul_fmatrix(FMatrix, FMatrix, FMatrix);
void mul_fmatrix_fvec(FVector, FMatrix, FVector);
void transpose_fmatrix(FMatrix, FMatrix);
void inv_fmatrix(FMatrix);

/* For DMatrix */
void add_dmatrix(DMatrix, DMatrix, DMatrix);
void sub_dmatrix(DMatrix, DMatrix, DMatrix);
void subst_dmatrix(DMatrix, DMatrix);
void set0_dmatrix(DMatrix);
void setI_dmatrix(DMatrix);
void cmul_dmatrix(DMatrix, double, DMatrix);
void mul_dmatrix(DMatrix, DMatrix, DMatrix);
void mul_dmatrix_dvec(DVector, DMatrix, DVector);
void transpose_dmatrix(DMatrix, DMatrix);
void inv_dmatrix(DMatrix);

/* For MPFMatrix */
#ifdef USE_GMP
void add_mpfmatrix(MPFMatrix, MPFMatrix, MPFMatrix);
void sub_mpfmatrix(MPFMatrix, MPFMatrix, MPFMatrix);
void subst_mpfmatrix(MPFMatrix, MPFMatrix);
void set0_mpfmatrix(MPFMatrix);
void setI_mpfmatrix(MPFMatrix);
void cmul_mpfmatrix(MPFMatrix, mpf_t, MPFMatrix);
void mul_mpfmatrix(MPFMatrix, MPFMatrix, MPFMatrix);
void mul_mpfmatrix_mpfvec(MPFVector, MPFMatrix, MPFVector);
void transpose_mpfmatrix(MPFMatrix, MPFMatrix);
void inv_mpfmatrix(MPFMatrix);
#endif

/*************************************************/
/* Linear System                                 */
/*************************************************/
/* LU decompositions & Backward Substitutions */
int FLUdecomp(FMatrix);
int SolveFLS(FVector, FMatrix, FVector);
int FLUdecompP(FMatrix, long[]);
int SolveFLSP(FVector, FMatrix, FVector, long[]);
int FLUdecompC(FMatrix, long[], long[]);
int SolveFLSC(FVector, FMatrix, FVector, long[], long[]);
int DLUdecomp(DMatrix);
int SolveDLS(DVector, DMatrix, DVector);
int DLUdecompP(DMatrix, long[]);
int SolveDLSP(DVector, DMatrix, DVector, long[]);
int DLUdecompC(DMatrix, long[], long[]);
int SolveDLSC(DVector, DMatrix, DVector, long[], long[]);
#ifdef USE_GMP
int MPFLUdecomp(MPFMatrix);
int SolveMPFLS(MPFVector, MPFMatrix, MPFVector);
int MPFLUdecompP(MPFMatrix, long[]);
int SolveMPFLSP(MPFVector, MPFMatrix, MPFVector, long[]);
int MPFLUdecompC(MPFMatrix, long[], long[]);
int SolveMPFLSC(MPFVector, MPFMatrix, MPFVector, long[], long[]);
#endif

/*************************************************/
/* cg.c: Conjugate-Gradient Methods              */
/*************************************************/
long int FCG(FVector, FMatrix, FVector, float, float, long);
long int DCG(DVector, DMatrix, DVector, double, double, long);
#ifdef USE_GMP
long int MPFCG(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
#endif

/*************************************************/
/* power.c, ivpower.c: Power and Inverse Power Methods */
/*************************************************/
long int fpower_eig(float *, FVector, FMatrix, FVector, float, float, long);
long int fivpower_eig(float *, FVector, FMatrix, FVector, float, float, long);
long int dpower_eig(double *, DVector, DMatrix, DVector, double, double, long);
long int divpower_eig(double *, DVector, DMatrix, DVector, double, double, long);
#ifdef USE_GMP
long int mpf_power_eig(mpf_t, MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
long int mpf_ivpower_eig(mpf_t, MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
#endif

/*************************************************/
/* newton.c: Newton/Simplified Newton Methods    */
/*************************************************/
long int fnewton_1(float *, float, float (* func)(float), float (* dfunc)(float), long int, float, float);
long int fsnewton_1(float *, float, float (* func)(float), float (* dfunc)(float), long int, float, float);
long int fnewton(FVector, FVector, void (* func)(FVector, FVector), void (* jfunc)(FMatrix, FVector), long int, float, float);
long int fsnewton(FVector, FVector, void (* func)(FVector, FVector), void (* jfunc)(FMatrix, FVector), long int, float, float);
long int dnewton_1(double *, double, double (* func)(double), double (* dfunc)(double), long int, double, double);
long int dsnewton_1(double *, double, double (* func)(double), double (* dfunc)(double), long int, double, double);
long int dnewton(DVector, DVector, void (* func)(DVector, DVector), void (* jfunc)(DMatrix, DVector), long int, double, double);
long int dsnewton(DVector, DVector, void (* func)(DVector, DVector), void (* jfunc)(DMatrix, DVector), long int, double, double);
#ifdef USE_GMP
long int mpf_newton_1(mpf_t, mpf_t, void (* func)(mpf_t, mpf_t), void (* dfunc)(mpf_t, mpf_t), long int, mpf_t, mpf_t);
long int mpf_snewton_1(mpf_t, mpf_t, void (* func)(mpf_t, mpf_t), void (* dfunc)(mpf_t, mpf_t), long int, mpf_t, mpf_t);
long int mpf_newton(MPFVector, MPFVector, void (* func)(MPFVector, MPFVector), void (* jfunc)(MPFMatrix, MPFVector), long int, mpf_t, mpf_t);
long int mpf_snewton(MPFVector, MPFVector, void (* func)(MPFVector, MPFVector), void (* jfunc)(MPFMatrix, MPFVector), long int, mpf_t, mpf_t);
#endif

/*************************************************/
/* poly.c: Algebraic equations and Polynomial    */
/*************************************************/
FPoly init_fpoly(long int);
DPoly init_dpoly(long int);
void free_fpoly(FPoly);
void free_dpoly(DPoly);
float get_fpoly_i(FPoly, long int);
double get_dpoly_i(DPoly, long int);
long int setdegree_fpoly(FPoly);
long int setdegree_dpoly(DPoly);
void set_fpoly_i(FPoly, long int, float);
void set_dpoly_i(DPoly, long int, double);
void print_fpoly(FPoly);
void print_dpoly(DPoly);
void add_fpoly(FPoly, FPoly, FPoly);
void sub_fpoly(FPoly, FPoly, FPoly);
void cmul_fpoly(FPoly, float, FPoly);
void subst_fpoly(FPoly, FPoly);
void set0_fpoly(FPoly);
long int max_abscoef_fpoly(FPoly);
long int num_nonzero_fpoly(FPoly);
void diff_fpoly(FPoly);
float eval_fpoly(FPoly, float);
float eval_diff_fpoly(FPoly, float);
void ceval_fpoly(FCmplx, FPoly, FCmplx);
void ceval_diff_fpoly(FCmplx, FPoly, FCmplx);
void add_dpoly(DPoly, DPoly, DPoly);
void sub_dpoly(DPoly, DPoly, DPoly);
void cmul_dpoly(DPoly, double, DPoly);
void subst_dpoly(DPoly, DPoly);
void set0_dpoly(DPoly);
long int max_abscoef_dpoly(DPoly);
long int num_nonzero_dpoly(DPoly);
void diff_dpoly(DPoly);
double eval_dpoly(DPoly, double);
double eval_diff_dpoly(DPoly, double);
void ceval_dpoly(DCmplx, DPoly, DCmplx);
void ceval_diff_dpoly(DCmplx, DPoly, DCmplx);
#ifdef USE_GMP
MPFPoly init_mpfpoly(long int);
MPFPoly init2_mpfpoly(long int, unsigned long);
void free_mpfpoly(MPFPoly);
mpf_ptr get_mpfpoly_i(MPFPoly, long int);
void set_mpfpoly_i(MPFPoly, long int, mpf_t);
void set_mpfpoly_i_d(MPFPoly, long int, double);
void set_mpfpoly_i_str(MPFPoly, long int, const char *, int);
void print_mpfpoly(MPFPoly);
void print_fdmpfpoly(FPoly, DPoly, MPFPoly);
void add_mpfpoly(MPFPoly, MPFPoly, MPFPoly);
void sub_mpfpoly(MPFPoly, MPFPoly, MPFPoly);
void cmul_mpfpoly(MPFPoly, mpf_t, MPFPoly);
void subst_mpfpoly(MPFPoly, MPFPoly);
void set0_mpfpoly(MPFPoly);
long int max_abscoef_mpfpoly(MPFPoly);
long int num_nonzero_mpfpoly(MPFPoly);
void diff_mpfpoly(MPFPoly);
void eval_mpfpoly(mpf_t, MPFPoly, mpf_t);
void eval_diff_mpfpoly(mpf_t, MPFPoly, mpf_t);
void ceval_mpfpoly(MPFCmplx, MPFPoly, MPFCmplx);
void ceval_diff_mpfpoly(MPFCmplx, MPFPoly, MPFCmplx);
#endif

/*************************************************/
/* complex.c: Complex arithmetic                 */
/*************************************************/
FCmplx init_fcmplx(void);
void free_fcmplx(FCmplx);
float get_real_fcmplx(FCmplx);
float get_image_fcmplx(FCmplx);
void set_real_fcmplx(FCmplx, float);
void set_image_fcmplx(FCmplx, float);
void subst_fcmplx(FCmplx, FCmplx);
void set0_fcmplx(FCmplx);
void add_fcmplx(FCmplx, FCmplx, FCmplx);
void add_fcmplx_f(FCmplx, FCmplx, float);
void add2_fcmplx(FCmplx, FCmplx);
void sub_fcmplx(FCmplx, FCmplx, FCmplx);
void conj_fcmplx(FCmplx, FCmplx);
void sign_fcmplx(FCmplx, FCmplx);
void sign2_fcmplx(FCmplx);
float abs_fcmplx(FCmplx);
void mul_fcmplx(FCmplx, FCmplx, FCmplx);
void mul_fcmplx_f(FCmplx, FCmplx, float);
void mul2_fcmplx(FCmplx, FCmplx);
void div_fcmplx(FCmplx, FCmplx, FCmplx);
void iexp_fcmplx(FCmplx, float);
void print_fcmplx(FCmplx);

DCmplx init_dcmplx(void);
void free_dcmplx(DCmplx);
double get_real_dcmplx(DCmplx);
double get_image_dcmplx(DCmplx);
void set_real_dcmplx(DCmplx, double);
void set_image_dcmplx(DCmplx, double);
void subst_dcmplx(DCmplx, DCmplx);
void set0_dcmplx(DCmplx);
void add_dcmplx(DCmplx, DCmplx, DCmplx);
void add_dcmplx_d(DCmplx, DCmplx, double);
void add2_dcmplx(DCmplx, DCmplx);
void sub_dcmplx(DCmplx, DCmplx, DCmplx);
void conj_dcmplx(DCmplx, DCmplx);
void sign_dcmplx(DCmplx, DCmplx);
void sign2_dcmplx(DCmplx);
double abs_dcmplx(DCmplx);
void mul_dcmplx(DCmplx, DCmplx, DCmplx);
void mul_dcmplx_d(DCmplx, DCmplx, double);
void mul2_dcmplx(DCmplx, DCmplx);
void div_dcmplx(DCmplx, DCmplx, DCmplx);
void iexp_dcmplx(DCmplx, double);
void print_dcmplx(DCmplx);

#ifdef USE_GMP
MPFCmplx init_mpfcmplx(void);
MPFCmplx init2_mpfcmplx(unsigned long int);
void free_mpfcmplx(MPFCmplx);
void get_real_mpfcmplx(mpf_t, MPFCmplx);
void get_image_mpfcmplx(mpf_t, MPFCmplx);
void set_real_mpfcmplx(MPFCmplx, mpf_t);
void set_image_mpfcmplx(MPFCmplx, mpf_t);
void set_real_mpfcmplx_ui(MPFCmplx, unsigned long int);
void set_image_mpfcmplx_ui(MPFCmplx, unsigned long int);
void subst_mpfcmplx(MPFCmplx, MPFCmplx);
void set0_mpfcmplx(MPFCmplx);
void add_mpfcmplx(MPFCmplx, MPFCmplx, MPFCmplx);
void add_mpfcmplx_mpf(MPFCmplx, MPFCmplx, mpf_t);
void add2_mpfcmplx(MPFCmplx, MPFCmplx);
void sub_mpfcmplx(MPFCmplx, MPFCmplx, MPFCmplx);
void conj_mpfcmplx(MPFCmplx, MPFCmplx);
void sign_mpfcmplx(MPFCmplx, MPFCmplx);
void sign2_mpfcmplx(MPFCmplx);
void abs_mpfcmplx(mpf_t, MPFCmplx);
void mul_mpfcmplx(MPFCmplx, MPFCmplx, MPFCmplx);
void mul_mpfcmplx_ui(MPFCmplx, MPFCmplx, unsigned long int);
void mul_mpfcmplx_mpf(MPFCmplx, MPFCmplx, mpf_t);
void mul2_mpfcmplx(MPFCmplx, MPFCmplx);
void div_mpfcmplx(MPFCmplx, MPFCmplx, MPFCmplx);
void iexp_mpfcmplx(MPFCmplx, mpf_t);
void print_mpfcmplx(MPFCmplx);
#endif


/*************************************************/
/* integral.c: Numerical Integral                */
/*************************************************/
float ftrapezoidal_fs(float, float, float(* func)(float), long int);
double dtrapezoidal_fs(double, double, double(* func)(double), long int);
#ifdef USE_GMP
void mpf_trapezoidal_fs(mpf_t, mpf_t, mpf_t, void(* func)(mpf_t, mpf_t), long int);
#endif

/*************************************************/
/* dka.c: DKA Methods                            */
/*************************************************/
float fdka_center(FPoly);
float fdka_radius(FPoly);
void fdka_init(CFArray, FPoly);
long int fdka(CFArray, CFArray, FPoly, long int, float, float);

double ddka_center(DPoly);
double ddka_radius(DPoly);
void ddka_init(CDArray, DPoly);
long int ddka(CDArray, CDArray, DPoly, long int, double, double);

#ifdef USE_GMP
void mpf_dka_center(mpf_t, MPFPoly);
void mpf_dka_radius(mpf_t, MPFPoly);
void mpf_dka_init(CMPFArray, MPFPoly);
long int mpf_dka(CMPFArray, CMPFArray, MPFPoly, long int, mpf_t, mpf_t);
#endif

/*****************************************************/
/* iterative.c: Iterative Methods for Linear Systems */
/*****************************************************/
void get_residual_fvector(FVector, FVector, FMatrix, FVector);
void fjacobi(FVector, FMatrix, FVector, float, float, long int);
void fgs(FVector, FMatrix, FVector, float, float, long int);
void fsor(FVector, FMatrix, FVector, float, float, float, long int);

void get_residual_dvector(DVector, DVector, DMatrix, DVector);
void djacobi(DVector, DMatrix, DVector, double, double, long int);
void dgs(DVector, DMatrix, DVector, double, double, long int);
void dsor(DVector, DMatrix, DVector, double, double, double, long int);

#ifdef USE_GMP
void mpf_jacobi(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long int);
void mpf_gs(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long int);
void mpf_sor(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, mpf_t, long int);
#endif

/************************************************/
/* qr.c: Gram-Schmidt, Modified Gram-Schmidt, QR*/
/************************************************/
void dgram_schmidt(DMatrix, DMatrix, DMatrix);
void dmgram_schmidt(DMatrix, DMatrix, DMatrix);
void dqr(DMatrix, long int);

#ifdef USE_GMP
void mpf_gram_schmidt(MPFMatrix, MPFMatrix, MPFMatrix);
void mpf_mgram_schmidt(MPFMatrix, MPFMatrix, MPFMatrix);
void mpf_qr(MPFMatrix, long int);
#endif

/************************************************/
/* gtestmat.c: Get Test Matrices                */
/************************************************/
void hilbert_dmatrix(DMatrix);
void lotkin_dmatrix(DMatrix);
void frank_dmatrix(DMatrix);
void tridiag_dmatrix(DMatrix, DVector, DVector, DVector);
void int_sym_rand_dmatrix(DMatrix, long int, long int);
void int_unsym_rand_dmatrix(DMatrix, long int, long int);
void diag_dmatrix(DMatrix, DVector);

#ifdef USE_GMP
void hilbert_mpfmatrix(MPFMatrix);
void lotkin_mpfmatrix(MPFMatrix);
void frank_mpfmatrix(MPFMatrix);
void tridiag_mpfmatrix(MPFMatrix, MPFVector, MPFVector, MPFVector);
void int_sym_rand_mpfmatrix(MPFMatrix, long int, long int);
void int_unsym_rand_mpfmatrix(MPFMatrix, long int, long int);
void diag_mpfmatrix(MPFMatrix, MPFVector);
#endif

#if defined (__cplusplus)
}
#endif

#endif /* End of __BNC_H */

