/************************************************/
/* Test program for euler.c & ex_nim.c          */
/* Copyleft 2000, Tomonori Kouya                */
/************************************************/
#include <stdio.h>

//#define USE_GMP
#include "bnc.h"

#ifndef USE_GMP
main()
{
	printf("This program is not available without GMP.\n");
}
#else

/* mpf_t */
void df(MPFVector y, mpf_t x0, MPFVector y0)
{
	/* y' = x + y */
	subst_mpfvector(y, y0);

	mpf_add(gmpfvi(y, 0), gmpfvi(y, 0), x0);

	return;
}

/* y = [exp(x), ...., exp(x)]^T */
void ans(MPFVector y, mpf_t x)
{
	long int i;
	mpf_t tmp;

	mpf_init(tmp);

	/* y(x) = 2e^x - x - 1 */
	mpf_exp(tmp, x);
	mpf_mul_ui(tmp, tmp, 2UL);
	mpf_sub(tmp, tmp, x);
	mpf_sub_ui(tmp, tmp, 1UL);

	set_mpfvector_i(y, 0, tmp);

	mpf_clear(tmp);

	return;
}


void initial_value(mpf_t lf_initx, mpf_t lf_maxx, MPFVector lf_inity)
{
	long int i;

	/* x0 := 0 */
	/* y := [1, ..., 1]^T */
	mpf_init_set_d(lf_initx, 0.0);
	mpf_init_set_d(lf_maxx, 10.0);
	for(i = 0; i < lf_inity->dim; i++)
		set_mpfvector_i_d(lf_inity, i, 1.0);
}

#include "mpf_erk_fs_coef.c"

#define DIM 1

main()
{
	MPFVector y, init_y, lf_tmp;
	MPFVector newcoef_c, newcoef_w;
	MPFMatrix newcoef_a;
	mpf_t tmp, x, init_x, stepsize, abs_tol, rel_tol;
	long int div_num;

	set_bnc_default_prec(128);

	/* x := 1 */
	mpf_init(init_x);
	mpf_init(x);
	y = init_mpfvector(DIM);
	lf_tmp = init_mpfvector(DIM);
	init_y = init_mpfvector(DIM);
	initial_value(init_x, x, init_y);

	mpf_init_set_d(stepsize, 0.5);
	mpf_init_set_d(abs_tol, 0.0);
	mpf_init_set_d(rel_tol, 1.0e-15);

	/* erk */
	printf("-- erk -- \n");

	/* set New Coefficients     */
	/*                          */
	/* 1/6 | 1/6                */
	/* 1/3 |  0  1/3            */
	/* 5/6 | 5/8 -5/3 15/8      */
	/* ----+----------------    */
	/*     | 1/10  0  5/10 4/10 */
    /*                          */

    newcoef_c = init_mpfvector(3);
    newcoef_w = init_mpfvector(4);
    newcoef_a = init_mpfmatrix(3,3);

	mpf_init_set_ui(tmp, 1UL);
	mpf_div_ui(tmp, tmp, 6UL);
	smpfvi(newcoef_c, 0, tmp);
	mpf_set_ui(tmp, 1UL);
	mpf_div_ui(tmp, tmp, 3UL);
	smpfvi(newcoef_c, 1, tmp);
	mpf_set_ui(tmp, 5UL);
	mpf_div_ui(tmp, tmp, 6UL);
	smpfvi(newcoef_c, 2, tmp);

	mpf_set_ui(tmp, 1UL);
	mpf_div_ui(tmp, tmp, 6UL);
	smpfmij(newcoef_a, 0, 0, tmp);
	mpf_set_ui(tmp, 0UL);
	smpfmij(newcoef_a, 1, 0, tmp);
	mpf_set_ui(tmp, 1UL);
	mpf_div_ui(tmp, tmp, 3UL);
	smpfmij(newcoef_a, 1, 1, tmp);
	mpf_set_ui(tmp, 5UL);
	mpf_div_ui(tmp, tmp, 8UL);
	smpfmij(newcoef_a, 2, 0, tmp);
	mpf_set_ui(tmp, 5UL); mpf_neg(tmp, tmp);
	mpf_div_ui(tmp, tmp, 3UL);
	smpfmij(newcoef_a, 2, 1, tmp);
	mpf_set_ui(tmp, 15UL);
	mpf_div_ui(tmp, tmp, 8UL);
	smpfmij(newcoef_a, 2, 2, tmp);

	mpf_set_ui(tmp, 1UL);
	mpf_div_ui(tmp, tmp, 10UL);
	smpfvi(newcoef_w, 0, tmp);
	mpf_set_ui(tmp, 0UL);
	smpfvi(newcoef_w, 1, tmp);
	mpf_set_ui(tmp, 5UL);
	mpf_div_ui(tmp, tmp, 10UL);
	smpfvi(newcoef_w, 2, tmp);
	mpf_set_ui(tmp, 4UL);
	mpf_div_ui(tmp, tmp, 10UL);
	smpfvi(newcoef_w, 3, tmp);

	initial_value(init_x, x, init_y);
	for(div_num = 2; div_num <= 1000; div_num *= 4)
	{
		mpf_erk_fs_coef(NULL, x, y, init_x, init_y, div_num, df, newcoef_c, newcoef_a, newcoef_w);
		ans(lf_tmp, x);
		sub_mpfvector(lf_tmp, lf_tmp, y);
		print_mpfvector(lf_tmp);
	}

	initial_value(init_x, x, init_y);
	for(div_num = 2; div_num <= 1000; div_num *= 4)
	{
		mpf_erk_fs     (NULL, x, y, init_x, init_y, div_num, df, ERK_CLASSICAL_4_4);
		ans(lf_tmp, x);
		sub_mpfvector(lf_tmp, lf_tmp, y);
		print_mpfvector(lf_tmp);
		printf("\n");
	}

	printf("-- erk ended -- \n");
#endif
}
